/*****************************************************************************/
/*                                                                           */
/*            SAS Program to Score the TIMSS 2023 4th Grade Items            */
/*                           TIMSS 2023 User Guide                           */
/*																			 */
/*             For use with all TIMSS 2023 files             				 */
/*                                                                           */
/*****************************************************************************/

%MACRO SCOREIT (ITEM, TYPE, RIGHT, NR, NA, OM, OTHER) ;

  * Code for multiple-choice items ;

  %IF %UPCASE(&TYPE) = "MC" %THEN %DO ;
    SCORE = 0 ;
      IF &ITEM = &RIGHT THEN SCORE = 1 ; * Item Key ;
      IF &ITEM = &NR    THEN SCORE = . ; * Not Reached ;
      IF &ITEM = &OM    THEN SCORE = . ; * Omitted ;
      IF &ITEM = &NA    THEN SCORE = . ; * Not Administered ;
      IF &ITEM = &OTHER THEN SCORE = 0 ; * Other Missing ;
    &ITEM = SCORE ;
  %END ;

  * Code for constructed-response items ;

  %IF %UPCASE(&TYPE) = "CR" %THEN %DO ;
    SCORE = 0 ;
	  IF (&ITEM = 2) OR (&ITEM >= 20 AND &ITEM < 30) THEN SCORE = 2 ; * Full Credit ;
      IF (&ITEM = 1) OR (&ITEM >= 10 AND &ITEM < 20) THEN SCORE = 1 ; * Full or Partial Credit ;
      IF (&ITEM = 0) OR (&ITEM >= 70 AND &ITEM < 80) THEN SCORE = 0 ; * No Credit ;
      IF &ITEM = &NR    THEN SCORE = . ; * Not Reached ;
      IF &ITEM = &OM    THEN SCORE = . ; * Omitted ;
      IF &ITEM = &NA    THEN SCORE = . ; * Not Administered ;
      IF &ITEM = &OTHER THEN SCORE = 0 ; * Other Missing ;
    &ITEM = SCORE ;
  %END ;

  DROP SCORE ;

%MEND SCOREIT ;

%MACRO DOIT (INDIR  = ,
             OUTDIR = ,
             INFILE = ) ;

  LIBNAME INDIR  "&INDIR" ;
  LIBNAME OUTDIR "&OUTDIR" ;

  OPTIONS NOTES MPRINT MPRINT FMTSEARCH=(INDIR,OUTDIR) ;

  * Get the student achievement data ;

  DATA OUTDIR.&INFILE._SCR; * Output File ;
       SET INDIR.&INFILE;   * Input File ;

       * Score multiple-choice items with A key ;

       ARRAY ARIGHT
             /* Digital  items */
             ME71199   ME71141A  ME71141D  ME71194B  ME61172    ME81359A  ME81359B 
			 ME81359D  ME81121   ME81218   ME81037   ME81003A   ME81003D  ME61232  
			 ME61095E  ME81054   ME81010A  ME61240A  ME61240B   ME61077   ME81065  
			 ME81057A  ME81057D  ME81112B  ME81112D  ME81112E   ME71013   ME71135A 
			 ME71005   ME71163   ME71213C  ME71187AB ME71138AA  ME71138AB ME71203  
			 SE71009D  SE71106A  SE71106D  SE71100   SE71220    SE71254A  SE71254B 
			 SE71254D  SE71254E  SE61048   SE61124B  SE61124D   SE61124E  SE61116B 
			 SE61116D  SE81083A  SE81083B  SE81083F  SE81083H   SE81004B  SE81004D 
			 SE81004E  SE81119A  SE81119D  SE81005B  SE81005D   SE81048E  SE81108C 
			 SE81115   SE81206   SE81070B  SE81070E  SE81013D   SE81055A  SE81055E 
			 SE81110B  SE81110D  SE71063B  SE71063E  SE71063F   SE71139   SE71114B 
			 SE71114D  SE71114F  SE71218   SE81014   SE81044A   SE81044C  SE81044D 
			 SE81042C  SE81401A  SE81401C  SE81118C  SE81118D   SE81239   SE71902B 
			 SE71902E  SE71046A  SE71046C  SE71129   SE71223    SE81012C  SE81081  
			 SE81238A  SE81053   SE81008   SE81056B  SE81030A   SE81030D  SE81030E 
			 SE81212C  SE81212D  SE61022   SE61160D  SE61160F   SE71910   SE71252  
			 SE71025   SE71145

             /* Paper items */
             MP71013   MP71141A  MP71141D  MP61172   MP71163   MP61232   MP61240A 
             MP61240B  MP61077   MP71138AA MP71138AB MP71203    
             SP71902B  SP71902E  SP71046A  SP71046C  SP71129   SP71025   SP71145  
             SP61022   SP71009D  SP71106A  SP71106D  SP71100   SP71220   SP61132  
             SP61042B  SP61155   SP71252   SP61048   SP61124B  SP61124D  SP61124E 
             SP61116B  SP61116D  SP71139   SP71218  

			 /* Less Difficult Mathematics items */
             MN21061   MN21037   MN21003A  MN21003D  MN11231   MN11045   MN11240  
			 MN11002   MN21054   MN21010A  MN11262   MN21065   MN21057A  MN21057D     ;
       DO OVER ARIGHT ; %SCOREIT (ARIGHT, "MC", 1, .R, .A, ., .I) ; END ;

       * Score multiple-choice items with B key ;

       ARRAY BRIGHT
             /* Digital items */
             ME71064   ME71176   ME71184   ME71141B  ME71141C  ME61275   ME81359C 
			 ME81359E  ME81359F  MQ81Z01   ME81067   ME81070   ME81003B  ME81003C 
			 ME61095C  ME61108   ME61211B  ME81038   ME81053   ME61240C  ME61041  
			 ME81035   ME81057B  ME81057C  ME81063   ME81012B  ME81360   ME81112A 
			 ME81112C  ME71178B  ME71213A  ME71213B  ME71070   ME71179A  ME71187AA
			 ME71094   ME71138AC     
			 SE71009C  SE71069   SE71103   SE71106B  SE71106C  SE71254C  SE71254F 
			 SE71254G  SE71254H  SE61047   SE61096   SE61124C  SE61116C  SE61116E 
			 SE81083D  SE81083E  SE81083G  SE81004A  SE81004F  SE81119B  SE81119C 
			 SE81150   SE81005A  SE81005C  SE81005E  SE81036   SE81048D  SE81152  
			 SE81108A  SE81070A  SE81070C  SE81070D  SE81055B  SE81055C  SE81055D 
			 SE81110A  SE81110E  SE81207B  SE71063A  SE71063C  SE71063D  SE71114A 
			 SE71114C  SE71114E  SE71152   SE81044B  SE81044E  SE81044F  SE81130  
			 SE81401B  SE81401E  SE81118A  SE81118B  SE81118E  SE71013   SE71902C 
			 SE71902D  SE71046B  SE71046D  SE71046E  SE71124   SE81080   SE81016  
			 SE81157   SE81238B  SE81030B  SE81030C  SE81102   SE81212B  SE81212E 
			 SE61140   SE61160B  SE61160E  SE61118   SE61097   SE71048   SE71901  
			 SE71201   

             /* Paper items */
             MP71178B  MP71064   MP71184   MP71141B  MP71141C  MP61275   MP71070  
			 MP71179A  MP61108   MP61211B  MP71062   MP71071   MP61240C  MP61041  
			 MP71094   MP71138AC     
			 SP71013   SP71902C  SP71902D  SP71046B  SP71046D  SP71046E  SP71124  
			 SP71223   SP71201   SP61140   SP61118   SP61097   SP71009C  SP71069  
			 SP71103   SP71106B  SP71106C  SP61042A  SP71048   SP71257   SP61047  
			 SP61096   SP61124C  SP61116C  SP61116E  SP71152  
      
			 /* Less Difficult Mathematics items */  
             MN21067   MN21070   MN21003B  MN21003C  MN11061   MN11134   MN11253  
			 MN11182B  MN21038   MN21053   MN11034   MN11239   MN21035   MN21057B 
			 MN21057C  MN21063   MN21012B     ;
       DO OVER BRIGHT ; %SCOREIT (BRIGHT, "MC", 2, .R, .A, ., .I) ; END ;

       * Score multiple-choice items with C key ;

       ARRAY CRIGHT
             /* Digital items */
			 ME71018   ME71083   ME71194A  ME61223   MQ71A05A  ME81069   ME81040  
			 ME61246   ME61049   ME61095B  ME61095D  ME81064   ME81030   ME61244  
			 ME81229   ME81039   ME71068   ME71178A  ME71178C  ME71045   ME71179B 
			 ME71179C  ME71187AD ME71079   ME71206     
			 SE71009B  SE71009F  SE71921   SE61125   SE61046   SE81039   SE81013B 
			 SE81110C  SE81207A  SE81209   SE71900   SE71118   SE81042B  SE81401D 
			 SE71076   SE71265   SE81012B  SE81155   SE81135   SE81200   SE81056A 
			 SE81077   SE81127   SE61135   SE61160C  SE71011   SE71127   SE71081  
			 SE71260       
      
             /* Paper items */     
             MP71068   MP71178A  MP71178C  MP71018   MP71083   MP61223   MP71045  
			 MP71179B  MP71179C  MP61246   MP61049   MP71134A  MP61244   MP71079  
			 MP71206        
			 SP71076   SP71265   SP71081   SP71260   SP61135   SP71009B  SP71009F 
			 SP61133A  SP71011   SP71127   SP61046   SP71900   SP71118   
      
			 /* Less Difficult  Mathematics items  */  
             MN21069   MN21040   MN11019   MN11211   MN11009   MN11182A  MN21064  
			 MN21030   MN11047A  MN21019   MN21039     ; 
       DO OVER CRIGHT ; %SCOREIT (CRIGHT, "MC", 3, .R, .A, ., .I) ; END ;

       * Score multiple-choice items with D key ;

       ARRAY DRIGHT
             /* Digital items */
			 ME61151   ME61269   ME81020   ME81001   ME81066   ME81215   ME61252  
			 ME81024   ME71040   ME71080   ME71187AC ME71008   ME71165   
			 SE71009A  SE71009E  SE71039   SE71080   SE61107   SE81048B  SE81108B 
			 SE81013A  SE81114   SE81137   SE71021   SE71214   SE81042A  SE81158  
			 SE81202   SE71112   SE81037   SE81012D  SE81049   SE81056C  SE61134  
			 SE71142   SE71500   SE71033   SE71144     
      
             /* Paper items */     
			 MP71040   MP71080   MP61151   MP61269   MP71001   MP61252   MP71008  
			 MP71165        
			 SP71112   SP71033   SP71144   SP61134   SP71009A  SP71009E  SP71039  
			 SP71080   SP61120   SP61133B  SP71142   SP71500   SP61107   SP71214  
      
			 /* Less Difficult  Mathemati s items  */  
             MN21020   MN21001   MN11265   MN11221   MN21066   MN21045   MN11047B 
			 MN21024     ;   
       DO OVER DRIGHT ; %SCOREIT (DRIGHT, "MC", 4, .R, .A, ., .I) ; END ;
 
       * Score multiple-choice items with E key ; 

       ARRAY ERIGHT
             /* Digital items */
             SE81048C SE81013C SE81012A   ;
       DO OVER ERIGHT ; %SCOREIT (ERIGHT, "MC", 5, .R, .A, ., .I) ; END ;

	   * Score multiple-choice items with F key ;

       ARRAY FRIGHT
             /* Digital items */
             SE81048A   ;
       DO OVER FRIGHT ; %SCOREIT (FRIGHT, "MC", 6, .R, .A, ., .I) ; END ;

       * Score Constructed-Response Items ;

       ARRAY CONSTR
             /* Digital items */
             ME71009   ME71037   ME71051   ME71169   ME71141   ME71194   ME71193  
			 ME71193A  ME71193B  ME71192   ME61027   ME61255   ME61043   ME61081  
			 ME61081A  ME61081B  ME81363   ME81363A  ME81363B  ME81359   MQ81Z02  
			 MQ81Z03   MQ81Z04   MQ81Z05   MQ81Z06   MQ81Z07   ME81315   ME81311  
			 MQ71A02A  MQ71A02B  MQ71A02C  MQ71A03A  MQ71A03B  MQ71A04A  MQ71A04AA
			 MQ71A04AB MQ71A04B  MQ71A05B  MQ71A05BA MQ71A05BB MQ71A06A  MQ71A06B 
			 MQ71R01A  MQ71R01B  MQ71R02A  MQ71R02B  MQ71R03   MQ71R03A  MQ71R03B 
			 MQ71R04   ME81364   ME81364A  ME81364B  ME81321   ME81900A  ME81900B 
			 ME81901   ME81314   ME81902   ME81350   ME81350A  ME81350B  ME81350C 
			 ME81350D  ME81338   ME81371A  ME81371B  ME81342   ME81322   ME81343  
			 ME81046   ME81023   ME81018   ME81033   ME81060   ME81060A  ME81060B 
			 ME81003   ME61178   ME61271   ME61256   ME61182   ME61095   ME61264  
			 ME61211A  ME81051   ME81051A  ME81051B  ME81025   ME81025A  ME81025B 
			 ME81025C  ME81043   ME81032   ME81010B  ME81059   ME61240   ME61254  
			 ME61254A  ME61254B  ME61254C  ME61173   ME61261   ME61224   ME61069A 
			 ME61069B  ME81049   ME81050   ME81234   ME81062   ME81057   ME81012A 
			 ME81012AA ME81012AB ME81332   ME81112   ME81323   MQ81B01   MQ81B05  
			 MQ81B05A  MQ81B05B  MQ81B06   MQ81B08   ME81334   ME81316   ME81352  
			 ME81324   ME81324A  ME81324B  ME71026   ME71036   ME71036A  ME71036B 
			 ME71075A  ME71075B  ME71211   ME71178   ME71135B  ME71201   ME71175  
			 ME71175A  ME71175B  ME71175C  ME71016   ME71213   ME71181   ME71179  
			 ME71067   ME71147A  ME71147B  ME71189   ME71187A  ME71187B  ME71024  
			 ME71049   ME71063   ME71081   ME71177   ME71138A  ME71138B  ME71205  
			 ME71205A  ME71205B  ME71205C     
			 SE71009   SE71093   SE71051   SE71137   SE71106   SE71254   SE61014  
			 SE61056   SE61015   SE61113   SE61124   SE61116   SE81083   SE81040  
			 SE81004   SE81073   SE81057   SE81054   SE81119   SQ81R01   SQ81R02  
			 SQ81R03   SQ81R04   SQ81R04A  SQ81R04B  SQ81R05   SQ81R05A  SQ81R05B 
			 SQ71S01   SQ71S02   SQ71S03   SQ71S03A  SQ71S03B  SQ71S04   SQ71S04A 
			 SQ71S04B  SQ71S05   SQ71S07   SQ71S08   SQ71S09   SE81005   SE81075  
			 SE81048   SE81035   SE81043   SE81108   SE81106   SE81920A  SE81920B 
			 SE81204   SE81070   SE81041   SE81007   SE81007A  SE81007B  SE81013  
			 SE81001   SE81055   SE81110   SE71063   SE71043   SE71005   SE71114  
			 SE71131   SE71213   SE81003   SE81003A  SE81003B  SE81044   SE81047  
			 SE81047A  SE81047B  SE81050   SE81042   SE81401   SE81118   SE81263  
			 SE71902   SE71041   SE71041A  SE71041B  SE71046   SE71095   SE71102  
			 SE81168   SE81012   SE81085   SE81085A  SE81085B  SE81238   SE81079  
			 SE81056   SE81030   SE81076   SE81124   SE81129   SE81212   SE81235  
			 SE61069   SE61019   SE61036   SE61160   SE61159   SE61091   SE71031  
			 SE71090   SE71071   SE71138   SE71222   SE71065   SE71056   SE71104  
			 SE71150   SE71237    
  
             /* Paper  items */
			 MP71026   MP71036   MP71036A  MP71036B  MP71075A  MP71075B  MP71211  
			 MP71178   MP71135B  MP71201   MP71175   MP71175A  MP71175B  MP71175C 
			 MP71009   MP71037   MP71051   MP71169   MP71141   MP71194   MP71193  
			 MP71193A  MP71193B  MP71192   MP61027   MP61255   MP61021   MP61043  
			 MP61081A  MP61081B  MP71016   MP71213   MP71181   MP71179   MP71067  
			 MP71147A  MP71147B  MP71189   MP71187B  MP71187A  MP61178   MP61271  
			 MP61256   MP61182   MP61095   MP61264   MP61211A  MP71010   MP71216A 
			 MP71216AA MP71216AB MP71216B  MP71117   MP71098   MP71134B  MP71202  
			 MP71190   MP71218   MP61240   MP61254   MP61173   MP61261   MP61224  
			 MP61069A  MP61069B  MP71024   MP71049   MP71063   MP71081   MP71177  
			 MP71138A  MP71138B  MP71205   MP71205A  MP71205B  MP71205C  
			 SP71902   SP71041   SP71041A  SP71041B  SP71046   SP71095   SP71102  
			 SP71065   SP71056   SP71104   SP71150   SP71237   SP61069   SP61019  
			 SP61036   SP61160   SP61159   SP61091   SP71009   SP71093   SP71051  
			 SP71137   SP71106   SP71254   SP61025   SP61074   SP61093   SP61161  
			 SP61041A  SP61041B  SP71031   SP71090   SP71071   SP71138   SP71222  
			 SP61014   SP61056   SP61015   SP61113   SP61124   SP61116   SP71063  
			 SP71043   SP71005   SP71114   SP71131   SP71213   
     
 			 /* Less Difficult Mathematics  items */   
             MN21046   MN21023   MN21018   MN21033   MN21060   MN21003   MN11145  
			 MN11014   MN11300   MN11028   MN11154   MN11024   MN11212   MN11146  
			 MN11177   MN11158   MN11272   MN21051   MN21025   MN21043   MN21032  
			 MN21010B  MN21059   MN11017   MN11125   MN11077   MN11223   MN11175  
			 MN11202   MN11299   MN21049   MN21050   MN21014   MN21062   MN21057  
			 MN21005   MN21012A     ;
       DO OVER CONSTR ; %SCOREIT (CONSTR, "CR",  , .R, .A, ., .I) ; END ;

  RUN ;

%MEND DOIT ;

%DOIT (INDIR  = <Specify location of input SAS data file using format x:\xxx> ,
       OUTDIR = <Specify location of output SAS data file using format x:\xxx> ,
       INFILE = <Specify name of merged SAS student achievement data file> ) ;
